###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("outline04.xlsx")

        self.ignore_files = [
            "xl/calcChain.xml",
            "[Content_Types].xml",
            "xl/_rels/workbook.xml.rels",
        ]

    def test_create_file(self):
        """
        Test the creation of a outlines in a XlsxWriter file. These tests are
        based on the outline programs in the examples directory.
        """

        workbook = Workbook(self.got_filename)

        worksheet4 = workbook.add_worksheet("Outline levels")

        levels = [
            "Level 1",
            "Level 2",
            "Level 3",
            "Level 4",
            "Level 5",
            "Level 6",
            "Level 7",
            "Level 6",
            "Level 5",
            "Level 4",
            "Level 3",
            "Level 2",
            "Level 1",
        ]

        worksheet4.write_column("A1", levels)

        worksheet4.set_row(0, None, None, {"level": 1})
        worksheet4.set_row(1, None, None, {"level": 2})
        worksheet4.set_row(2, None, None, {"level": 3})
        worksheet4.set_row(3, None, None, {"level": 4})
        worksheet4.set_row(4, None, None, {"level": 5})
        worksheet4.set_row(5, None, None, {"level": 6})
        worksheet4.set_row(6, None, None, {"level": 7})
        worksheet4.set_row(7, None, None, {"level": 6})
        worksheet4.set_row(8, None, None, {"level": 5})
        worksheet4.set_row(9, None, None, {"level": 4})
        worksheet4.set_row(10, None, None, {"level": 3})
        worksheet4.set_row(11, None, None, {"level": 2})
        worksheet4.set_row(12, None, None, {"level": 1})

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...workbook import Workbook
from ...exceptions import OverlappingRange


class TestOverlapRanges(unittest.TestCase):
    """
    Test overlapping merge and table ranges.

    """

    def setUp(self):
        fh = StringIO()
        workbook = Workbook()
        workbook._set_filehandle(fh)
        self.workbook = workbook

    def test_overlaps01(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.merge_range("A1:G10", "")

        with self.assertRaises(OverlappingRange):
            worksheet.merge_range("A1:G10", "")

    def test_overlaps02(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.merge_range("A1:G10", "")

        with self.assertRaises(OverlappingRange):
            worksheet.merge_range("B3:C3", "")

    def test_overlaps03(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.merge_range("A1:G10", "")

        with self.assertRaises(OverlappingRange):
            worksheet.merge_range("G10:G11", "")

    def test_overlaps04(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.add_table("A1:G10")

        with self.assertRaises(OverlappingRange):
            worksheet.add_table("A1:G10")

    def test_overlaps05(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.add_table("A1:G10")

        with self.assertRaises(OverlappingRange):
            worksheet.add_table("B3:C3")

    def test_overlaps06(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.add_table("A1:G10")

        with self.assertRaises(OverlappingRange):
            worksheet.add_table("G1:G11")

    def test_overlaps07(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.merge_range("A1:G10", "")

        with self.assertRaises(OverlappingRange):
            worksheet.add_table("B3:C3")

    def test_overlaps08(self):
        """Test Worksheet range overlap exceptions"""
        worksheet = self.workbook.add_worksheet()

        worksheet.add_table("A1:G10")

        with self.assertRaises(OverlappingRange):
            worksheet.merge_range("B3:C3", "")



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("table26.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file with tables."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        worksheet.set_column("C:D", 10.288)
        worksheet.set_column("F:G", 10.288)

        worksheet.add_table("C2:D3")
        worksheet.add_table("F3:G3", {"header_row": 0})

        # These tables should be ignored since the ranges are incorrect.
        import warnings

        warnings.filterwarnings("ignore")
        worksheet.add_table("I2:J2")
        worksheet.add_table("L3:M3", {"header_row": 1})

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestWritePageMargins(unittest.TestCase):
    """
    Test the Worksheet _write_page_margins() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_page_margins(self):
        """Test the _write_page_margins() method"""

        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_deafult(self):
        """Test the _write_page_margins() method with default margins"""

        self.worksheet.set_margins()
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_left(self):
        """Test the _write_page_margins() method with left margin"""

        self.worksheet.set_margins(left=0.5)
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.5" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_right(self):
        """Test the _write_page_margins() method with right margin"""

        self.worksheet.set_margins(right=0.5)
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.5" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_top(self):
        """Test the _write_page_margins() method with top margin"""

        self.worksheet.set_margins(top=0.5)
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.7" top="0.5" bottom="0.75" header="0.3" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_bottom(self):
        """Test the _write_page_margins() method with bottom margin"""

        self.worksheet.set_margins(bottom=0.5)
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.7" top="0.75" bottom="0.5" header="0.3" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_header(self):
        """Test the _write_page_margins() method with header margin"""

        self.worksheet.set_header(margin=0.5)
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.5" footer="0.3"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_page_margins_footer(self):
        """Test the _write_page_margins() method with footer margin"""

        self.worksheet.set_footer(margin=0.5)
        self.worksheet._write_page_margins()

        exp = """<pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.5"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...xmlwriter import XMLwriter


class TestXMLwriter(unittest.TestCase):
    """
    Test the XML Writer class.

    """

    def setUp(self):
        self.fh = StringIO()
        self.writer = XMLwriter()
        self.writer._set_filehandle(self.fh)

    def test_xml_declaration(self):
        """Test _xml_declaration()"""

        self.writer._xml_declaration()

        exp = """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>\n"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_start_tag(self):
        """Test _xml_start_tag() with no attributes"""

        self.writer._xml_start_tag("foo")

        exp = """<foo>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_start_tag_with_attributes(self):
        """Test _xml_start_tag() with attributes"""

        self.writer._xml_start_tag("foo", [("span", "8"), ("baz", "7")])

        exp = """<foo span="8" baz="7">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_start_tag_with_attributes_to_escape(self):
        """Test _xml_start_tag() with attributes requiring escaping"""

        self.writer._xml_start_tag("foo", [("span", '&<>"')])

        exp = """<foo span="&amp;&lt;&gt;&quot;">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_start_tag_unencoded(self):
        """Test _xml_start_tag_unencoded() with attributes"""

        self.writer._xml_start_tag_unencoded("foo", [("span", '&<>"')])

        exp = """<foo span="&<>"">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_end_tag(self):
        """Test _xml_end_tag()"""

        self.writer._xml_end_tag("foo")

        exp = """</foo>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_empty_tag(self):
        """Test _xml_empty_tag()"""

        self.writer._xml_empty_tag("foo")

        exp = """<foo/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_empty_tag_with_attributes(self):
        """Test _xml_empty_tag() with attributes"""

        self.writer._xml_empty_tag("foo", [("span", "8")])

        exp = """<foo span="8"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_empty_tag_unencoded(self):
        """Test _xml_empty_tag_unencoded() with attributes"""

        self.writer._xml_empty_tag_unencoded("foo", [("span", "&")])

        exp = """<foo span="&"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_data_element(self):
        """Test _xml_data_element()"""

        self.writer._xml_data_element("foo", "bar")

        exp = """<foo>bar</foo>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_data_element_with_attributes(self):
        """Test _xml_data_element() with attributes"""

        self.writer._xml_data_element("foo", "bar", [("span", "8")])

        exp = """<foo span="8">bar</foo>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_data_element_with_escapes(self):
        """Test _xml_data_element() with data requiring escaping"""

        self.writer._xml_data_element("foo", '&<>"', [("span", "8")])

        exp = """<foo span="8">&amp;&lt;&gt;"</foo>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_string_element(self):
        """Test _xml_string_element()"""

        self.writer._xml_string_element(99, [("span", "8")])

        exp = """<c span="8" t=\"s\"><v>99</v></c>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_si_element(self):
        """Test _xml_si_element()"""

        self.writer._xml_si_element("foo", [("span", "8")])

        exp = """<si><t span="8">foo</t></si>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_rich_si_element(self):
        """Test _xml_rich_si_element()"""

        self.writer._xml_rich_si_element("foo")

        exp = """<si>foo</si>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_number_element(self):
        """Test _xml_number_element()"""

        self.writer._xml_number_element(99, [("span", "8")])

        exp = """<c span="8"><v>99</v></c>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_xml_formula_element(self):
        """Test _xml_formula_element()"""

        self.writer._xml_formula_element("1+2", 3, [("span", "8")])

        exp = """<c span="8"><f>1+2</f><v>3</v></c>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...workbook import Workbook


class TestWriteWorkbookView(unittest.TestCase):
    """
    Test the Workbook _write_workbook_view() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.workbook = Workbook()
        self.workbook._set_filehandle(self.fh)

    def test_write_workbook_view1(self):
        """Test the _write_workbook_view() method"""

        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view2(self):
        """Test the _write_workbook_view() method"""

        self.workbook.worksheet_meta.activesheet = 1
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660" activeTab="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view3(self):
        """Test the _write_workbook_view() method"""

        self.workbook.worksheet_meta.firstsheet = 1
        self.workbook.worksheet_meta.activesheet = 1
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660" firstSheet="2" activeTab="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view4(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_size(0, 0)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view5(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_size(None, None)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view6(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_size(1073, 644)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view7(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_size(123, 70)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="1845" windowHeight="1050"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view8(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_size(719, 490)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="10785" windowHeight="7350"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view9(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_tab_ratio()
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view10(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_tab_ratio(34.6)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660" tabRatio="346"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view11(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_tab_ratio(0)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660" tabRatio="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_workbook_view12(self):
        """Test the _write_workbook_view() method"""

        self.workbook.set_tab_ratio(100)
        self.workbook._write_workbook_view()

        exp = """<workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660" tabRatio="1000"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def tearDown(self):
        self.workbook.fileclosed = 1



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from datetime import date
from datetime import datetime
from datetime import time
from datetime import timedelta
from datetime import tzinfo
from ...workbook import Workbook


# Simple class to add timezone to dates for testing.
class GMT(tzinfo):
    def utcoffset(self, dt):
        return timedelta(hours=1)

    def dst(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return "Europe"


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("remove_timezone01.xlsx")

    def test_remove_timezone_none(self):
        """Test write_datetime without timezones."""

        workbook = Workbook(self.got_filename, {"remove_timezone": False})

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 20)

        format1 = workbook.add_format({"num_format": 20})
        format2 = workbook.add_format({"num_format": 14})
        format3 = workbook.add_format({"num_format": 22})

        date1 = time(12, 0, 0)
        date2 = date(2016, 9, 23)
        date3 = datetime.strptime("2016-09-12 12:00", "%Y-%m-%d %H:%M")

        worksheet.write_datetime(0, 0, date1, format1)
        worksheet.write_datetime(1, 0, date2, format2)
        worksheet.write_datetime(2, 0, date3, format3)

        workbook.close()

        self.assertExcelEqual()

    def test_remove_timezone_gmt(self):
        """Test write_datetime with timezones."""

        workbook = Workbook(self.got_filename, {"remove_timezone": True})

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 20)

        format1 = workbook.add_format({"num_format": 20})
        format2 = workbook.add_format({"num_format": 14})
        format3 = workbook.add_format({"num_format": 22})

        date1 = time(12, 0, 0, tzinfo=GMT())
        date2 = date(2016, 9, 23)
        date3 = datetime.strptime("2016-09-12 12:00", "%Y-%m-%d %H:%M")

        date3 = date3.replace(tzinfo=GMT())

        worksheet.write_datetime(1, 0, date2, format2)
        worksheet.write_datetime(2, 0, date3, format3)

        workbook.close()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...workbook import Workbook


class TestWriteDefinedNames(unittest.TestCase):
    """
    Test the Workbook _write_defined_names() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.workbook = Workbook()
        self.workbook._set_filehandle(self.fh)

    def test_write_defined_names_1(self):
        """Test the _write_defined_names() method"""

        self.workbook.defined_names = [["_xlnm.Print_Titles", 0, "Sheet1!$1:$1", 0]]

        self.workbook._write_defined_names()

        exp = """<definedNames><definedName name="_xlnm.Print_Titles" localSheetId="0">Sheet1!$1:$1</definedName></definedNames>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_defined_names_2(self):
        """Test the _write_defined_names() method"""

        self.workbook.add_worksheet()
        self.workbook.add_worksheet()
        self.workbook.add_worksheet("Sheet 3")

        self.workbook.define_name("""'Sheet 3'!Bar""", """='Sheet 3'!$A$1""")
        self.workbook.define_name("""Abc""", """=Sheet1!$A$1""")
        self.workbook.define_name("""Baz""", """=0.98""")
        self.workbook.define_name("""Sheet1!Bar""", """=Sheet1!$A$1""")
        self.workbook.define_name("""Sheet2!Bar""", """=Sheet2!$A$1""")
        self.workbook.define_name("""Sheet2!aaa""", """=Sheet2!$A$1""")
        self.workbook.define_name("""'Sheet 3'!car""", '="Saab 900"')
        self.workbook.define_name("""_Egg""", """=Sheet1!$A$1""")
        self.workbook.define_name("""_Fog""", """=Sheet1!$A$1""")

        self.workbook._prepare_defined_names()
        self.workbook._write_defined_names()

        exp = """<definedNames><definedName name="_Egg">Sheet1!$A$1</definedName><definedName name="_Fog">Sheet1!$A$1</definedName><definedName name="aaa" localSheetId="1">Sheet2!$A$1</definedName><definedName name="Abc">Sheet1!$A$1</definedName><definedName name="Bar" localSheetId="2">'Sheet 3'!$A$1</definedName><definedName name="Bar" localSheetId="0">Sheet1!$A$1</definedName><definedName name="Bar" localSheetId="1">Sheet2!$A$1</definedName><definedName name="Baz">0.98</definedName><definedName name="car" localSheetId="2">"Saab 900"</definedName></definedNames>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def tearDown(self):
        self.workbook.fileclosed = 1



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...app import App


class TestAssembleApp(unittest.TestCase):
    """
    Test assembling a complete App file.

    """

    def test_assemble_xml_file(self):
        """Test writing an App file."""
        self.maxDiff = None

        fh = StringIO()
        app = App()
        app._set_filehandle(fh)

        app._add_part_name("Sheet1")
        app._add_heading_pair(("Worksheets", 1))

        app._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <Properties xmlns="http://schemas.openxmlformats.org/officeDocument/2006/extended-properties" xmlns:vt="http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes">
                  <Application>Microsoft Excel</Application>
                  <DocSecurity>0</DocSecurity>
                  <ScaleCrop>false</ScaleCrop>
                  <HeadingPairs>
                    <vt:vector size="2" baseType="variant">
                      <vt:variant>
                        <vt:lpstr>Worksheets</vt:lpstr>
                      </vt:variant>
                      <vt:variant>
                        <vt:i4>1</vt:i4>
                      </vt:variant>
                    </vt:vector>
                  </HeadingPairs>
                  <TitlesOfParts>
                    <vt:vector size="1" baseType="lpstr">
                      <vt:lpstr>Sheet1</vt:lpstr>
                    </vt:vector>
                  </TitlesOfParts>
                  <Company>
                  </Company>
                  <LinksUpToDate>false</LinksUpToDate>
                  <SharedDoc>false</SharedDoc>
                  <HyperlinksChanged>false</HyperlinksChanged>
                  <AppVersion>12.0000</AppVersion>
                </Properties>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("image09.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file with image(s)."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        worksheet.insert_image("E9", self.image_dir + "red_64x20.png")

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestWriteSheetViews(unittest.TestCase):
    """
    Test the Worksheet _write_sheet_views() method.
    Repeat of tests in test_write_sheet_views3.py with explicit topLeft cells.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_sheet_views1(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(15, 0, 1, 0)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane ySplit="600" topLeftCell="A2"/><selection pane="bottomLeft" activeCell="A2" sqref="A2"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views2(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(30, 0, 2, 0)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane ySplit="900" topLeftCell="A3"/><selection pane="bottomLeft" activeCell="A3" sqref="A3"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views3(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(105, 0, 7, 0)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane ySplit="2400" topLeftCell="A8"/><selection pane="bottomLeft" activeCell="A8" sqref="A8"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views4(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(0, 8.43, 0, 1)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane xSplit="1350" topLeftCell="B1"/><selection pane="topRight" activeCell="B1" sqref="B1"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views5(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(0, 17.57, 0, 2)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane xSplit="2310" topLeftCell="C1"/><selection pane="topRight" activeCell="C1" sqref="C1"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views6(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(0, 45, 0, 5)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane xSplit="5190" topLeftCell="F1"/><selection pane="topRight" activeCell="F1" sqref="F1"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views7(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(15, 8.43, 1, 1)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane xSplit="1350" ySplit="600" topLeftCell="B2"/><selection pane="topRight" activeCell="B1" sqref="B1"/><selection pane="bottomLeft" activeCell="A2" sqref="A2"/><selection pane="bottomRight" activeCell="B2" sqref="B2"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_views8(self):
        """Test the _write_sheet_views() method with split panes"""

        self.worksheet.select()

        self.worksheet.split_panes(45, 54.14, 3, 6)

        self.worksheet._write_sheet_views()

        exp = '<sheetViews><sheetView tabSelected="1" workbookViewId="0"><pane xSplit="6150" ySplit="1200" topLeftCell="G4"/><selection pane="topRight" activeCell="G1" sqref="G1"/><selection pane="bottomLeft" activeCell="A4" sqref="A4"/><selection pane="bottomRight" activeCell="G4" sqref="G4"/></sheetView></sheetViews>'
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...styles import Styles
from ...format import Format


class TestWriteBorders(unittest.TestCase):
    """
    Test the Styles _write_borders() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.styles = Styles()
        self.styles._set_filehandle(self.fh)

    def test_write_borders(self):
        """Test the _write_borders() method"""

        xf_format = Format()
        xf_format.has_border = 1

        self.styles._set_style_properties([[xf_format], None, 0, 0, 1, 0, [], [], 0])

        self.styles._write_borders()

        exp = """<borders count="1"><border><left/><right/><top/><bottom/><diagonal/></border></borders>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...chartsheet import Chartsheet


class TestWriteSheetProtection(unittest.TestCase):
    """
    Test the Chartsheet _write_sheet_protection() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.chartsheet = Chartsheet()
        self.chartsheet._set_filehandle(self.fh)

    def test_write_sheet_protection_1(self):
        """Test the _write_sheet_protection() method."""

        password = ""
        options = {}

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = """<sheetProtection content="1" objects="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_protection_2(self):
        """Test the _write_sheet_protection() method."""

        password = "password"
        options = {}

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = """<sheetProtection password="83AF" content="1" objects="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_protection_3(self):
        """Test the _write_sheet_protection() method."""

        password = ""
        options = {"objects": False}

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = """<sheetProtection content="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_protection_4(self):
        """Test the _write_sheet_protection() method."""

        password = ""
        options = {"content": False}

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = """<sheetProtection objects="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_protection_5(self):
        """Test the _write_sheet_protection() method."""

        password = ""
        options = {"content": False, "objects": False}

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = ""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_protection_6(self):
        """Test the _write_sheet_protection() method."""

        password = "password"
        options = {"content": False, "objects": False}

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = """<sheetProtection password="83AF"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_protection_7(self):
        """Test the _write_sheet_protection() method."""

        password = "password"
        options = {
            "objects": True,
            "scenarios": True,
            "format_cells": True,
            "format_columns": True,
            "format_rows": True,
            "insert_columns": True,
            "insert_rows": True,
            "insert_hyperlinks": True,
            "delete_columns": True,
            "delete_rows": True,
            "select_locked_cells": False,
            "sort": True,
            "autofilter": True,
            "pivot_tables": True,
            "select_unlocked_cells": False,
        }

        self.chartsheet.protect(password, options)
        self.chartsheet._write_sheet_protection()

        exp = """<sheetProtection password="83AF" content="1" objects="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list, _vml_to_list
from ...vml import Vml


class TestAssembleVml(unittest.TestCase):
    """
    Test assembling a complete Vml file.

    """

    def test_assemble_xml_file(self):
        """Test writing a vml with no cell data."""
        self.maxDiff = None

        fh = StringIO()
        vml = Vml()
        vml._set_filehandle(fh)

        vml._assemble_xml_file(1, 1024, None, None, [[32, 32, "red", "CH", 96, 96, 1]])

        exp = _vml_to_list(
            """
                <xml xmlns:v="urn:schemas-microsoft-com:vml" xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel">
                  <o:shapelayout v:ext="edit">
                    <o:idmap v:ext="edit" data="1"/>
                  </o:shapelayout>
                  <v:shapetype id="_x0000_t75" coordsize="21600,21600" o:spt="75" o:preferrelative="t" path="m@4@5l@4@11@9@11@9@5xe" filled="f" stroked="f">
                    <v:stroke joinstyle="miter"/>
                    <v:formulas>
                      <v:f eqn="if lineDrawn pixelLineWidth 0"/>
                      <v:f eqn="sum @0 1 0"/>
                      <v:f eqn="sum 0 0 @1"/>
                      <v:f eqn="prod @2 1 2"/>
                      <v:f eqn="prod @3 21600 pixelWidth"/>
                      <v:f eqn="prod @3 21600 pixelHeight"/>
                      <v:f eqn="sum @0 0 1"/>
                      <v:f eqn="prod @6 1 2"/>
                      <v:f eqn="prod @7 21600 pixelWidth"/>
                      <v:f eqn="sum @8 21600 0"/>
                      <v:f eqn="prod @7 21600 pixelHeight"/>
                      <v:f eqn="sum @10 21600 0"/>
                    </v:formulas>
                    <v:path o:extrusionok="f" gradientshapeok="t" o:connecttype="rect"/>
                    <o:lock v:ext="edit" aspectratio="t"/>
                  </v:shapetype>
                  <v:shape id="CH" o:spid="_x0000_s1025" type="#_x0000_t75" style="position:absolute;margin-left:0;margin-top:0;width:24pt;height:24pt;z-index:1">
                    <v:imagedata o:relid="rId1" o:title="red"/>
                    <o:lock v:ext="edit" rotation="t"/>
                  </v:shape>
                </xml>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestWriteSheetView(unittest.TestCase):
    """
    Test the Worksheet _write_sheet_view() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_sheet_view_tab_not_selected(self):
        """Test the _write_sheet_view() method. Tab not selected"""

        self.worksheet._write_sheet_view()

        exp = """<sheetView workbookViewId="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_view_tab_selected(self):
        """Test the _write_sheet_view() method. Tab selected"""

        self.worksheet.select()
        self.worksheet._write_sheet_view()

        exp = """<sheetView tabSelected="1" workbookViewId="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_view_hide_gridlines(self):
        """Test the _write_sheet_view() method. Tab selected + hide_gridlines()"""

        self.worksheet.select()
        self.worksheet.hide_gridlines()
        self.worksheet._write_sheet_view()

        exp = """<sheetView tabSelected="1" workbookViewId="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_view_hide_gridlines_0(self):
        """Test the _write_sheet_view() method. Tab selected + hide_gridlines(0)"""

        self.worksheet.select()
        self.worksheet.hide_gridlines(0)
        self.worksheet._write_sheet_view()

        exp = """<sheetView tabSelected="1" workbookViewId="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_view_hide_gridlines_1(self):
        """Test the _write_sheet_view() method. Tab selected + hide_gridlines(1)"""

        self.worksheet.select()
        self.worksheet.hide_gridlines(1)
        self.worksheet._write_sheet_view()

        exp = """<sheetView tabSelected="1" workbookViewId="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_view_hide_gridlines_2(self):
        """Test the _write_sheet_view() method. Tab selected + hide_gridlines(2)"""

        self.worksheet.select()
        self.worksheet.hide_gridlines(2)
        self.worksheet._write_sheet_view()

        exp = """<sheetView showGridLines="0" tabSelected="1" workbookViewId="0"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet_view_hide_row_col_headers(self):
        """Test the _write_sheet_views() method"""

        self.worksheet.select()
        self.worksheet.hide_row_col_headers()
        self.worksheet._write_sheet_view()

        exp = (
            """<sheetView showRowColHeaders="0" tabSelected="1" workbookViewId="0"/>"""
        )
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...workbook import Workbook


class TestWriteSheet(unittest.TestCase):
    """
    Test the Workbook _write_sheet() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.workbook = Workbook()
        self.workbook._set_filehandle(self.fh)

    def test_write_sheet1(self):
        """Test the _write_sheet() method"""

        self.workbook._write_sheet("Sheet1", 1, 0)

        exp = """<sheet name="Sheet1" sheetId="1" r:id="rId1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet2(self):
        """Test the _write_sheet() method"""

        self.workbook._write_sheet("Sheet1", 1, 1)

        exp = """<sheet name="Sheet1" sheetId="1" state="hidden" r:id="rId1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_sheet3(self):
        """Test the _write_sheet() method"""

        self.workbook._write_sheet("Bits & Bobs", 1, 0)

        exp = """<sheet name="Bits &amp; Bobs" sheetId="1" r:id="rId1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def tearDown(self):
        self.workbook.fileclosed = 1



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("optimize03.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(
            self.got_filename, {"constant_memory": True, "in_memory": False}
        )
        worksheet = workbook.add_worksheet()

        bold = workbook.add_format({"bold": 1})

        worksheet.set_column("A:A", 36, bold)
        worksheet.set_column("B:B", 20)
        worksheet.set_row(0, 40)

        heading_format = workbook.add_format(
            {
                "bold": 1,
                "font_color": "blue",
                "font_size": 16,
                "align": "centre_across",
                "valign": "vcenter",
            }
        )

        heading_format.text_h_align = 6

        hyperlink_format = workbook.add_format(
            {
                "font_color": "blue",
                "underline": 1,
            }
        )

        headings = ["Features of Excel::Writer::XLSX", ""]
        worksheet.write_row("A1", headings, heading_format)

        text_format = workbook.add_format(
            {
                "bold": 1,
                "italic": 1,
                "font_color": "red",
                "font_size": 18,
                "font": "Lucida Calligraphy",
            }
        )

        worksheet.write("A2", "Text")
        worksheet.write("B2", "Hello Excel")
        worksheet.write("A3", "Formatted text")
        worksheet.write("B3", "Hello Excel", text_format)

        num1_format = workbook.add_format({"num_format": "$#,##0.00"})
        num2_format = workbook.add_format({"num_format": " d mmmm yyy"})

        worksheet.write("A5", "Numbers")
        worksheet.write("B5", 1234.56)
        worksheet.write("A6", "Formatted numbers")
        worksheet.write("B6", 1234.56, num1_format)
        worksheet.write("A7", "Formatted numbers")
        worksheet.write("B7", 37257, num2_format)

        worksheet.write("A8", 'Formulas and functions, "=SIN(PI()/4)"')
        worksheet.write("B8", "=SIN(PI()/4)")

        worksheet.write("A9", "Hyperlinks")
        worksheet.write("B9", "http://www.perl.com/", hyperlink_format)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("outline06.xlsx")

        self.ignore_files = [
            "xl/calcChain.xml",
            "[Content_Types].xml",
            "xl/_rels/workbook.xml.rels",
        ]

    def test_create_file(self):
        """
        Test the creation of a outlines in a XlsxWriter file. These tests are
        based on the outline programs in the examples directory.
        """

        workbook = Workbook(self.got_filename)

        worksheet1 = workbook.add_worksheet("Outlined Rows")

        worksheet1.outline_settings(0, 0, 0, 1)

        bold = workbook.add_format({"bold": 1})

        worksheet1.set_row(1, None, None, {"level": 2})
        worksheet1.set_row(2, None, None, {"level": 2})
        worksheet1.set_row(3, None, None, {"level": 2})
        worksheet1.set_row(4, None, None, {"level": 2})
        worksheet1.set_row(5, None, None, {"level": 1})

        worksheet1.set_row(6, None, None, {"level": 2})
        worksheet1.set_row(7, None, None, {"level": 2})
        worksheet1.set_row(8, None, None, {"level": 2})
        worksheet1.set_row(9, None, None, {"level": 2})
        worksheet1.set_row(10, None, None, {"level": 1})

        worksheet1.set_column("A:A", 20)

        worksheet1.write("A1", "Region", bold)
        worksheet1.write("A2", "North")
        worksheet1.write("A3", "North")
        worksheet1.write("A4", "North")
        worksheet1.write("A5", "North")
        worksheet1.write("A6", "North Total", bold)

        worksheet1.write("B1", "Sales", bold)
        worksheet1.write("B2", 1000)
        worksheet1.write("B3", 1200)
        worksheet1.write("B4", 900)
        worksheet1.write("B5", 1200)
        worksheet1.write("B6", "=SUBTOTAL(9,B2:B5)", bold, 4300)

        worksheet1.write("A7", "South")
        worksheet1.write("A8", "South")
        worksheet1.write("A9", "South")
        worksheet1.write("A10", "South")
        worksheet1.write("A11", "South Total", bold)

        worksheet1.write("B7", 400)
        worksheet1.write("B8", 600)
        worksheet1.write("B9", 500)
        worksheet1.write("B10", 600)
        worksheet1.write("B11", "=SUBTOTAL(9,B7:B10)", bold, 2100)

        worksheet1.write("A12", "Grand Total", bold)
        worksheet1.write("B12", "=SUBTOTAL(9,B2:B10)", bold, 6400)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...workbook import Workbook


class TestAssembleWorkbook(unittest.TestCase):
    """
    Test assembling a complete Workbook file.

    """

    def test_assemble_xml_file(self):
        """Test writing a workbook with 2 worksheets."""
        self.maxDiff = None

        fh = StringIO()
        workbook = Workbook()
        workbook._set_filehandle(fh)

        workbook.add_worksheet()
        workbook.add_worksheet()

        workbook._assemble_xml_file()
        workbook.fileclosed = 1

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <workbook xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
                  <fileVersion appName="xl" lastEdited="4" lowestEdited="4" rupBuild="4505"/>
                  <workbookPr defaultThemeVersion="124226"/>
                  <bookViews>
                    <workbookView xWindow="240" yWindow="15" windowWidth="16095" windowHeight="9660"/>
                  </bookViews>
                  <sheets>
                    <sheet name="Sheet1" sheetId="1" r:id="rId1"/>
                    <sheet name="Sheet2" sheetId="2" r:id="rId2"/>
                  </sheets>
                  <calcPr calcId="124519" fullCalcOnLoad="1"/>
                </workbook>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("simple02.xlsx")

    def test_create_file(self):
        """Test the creation of a simple workbook."""

        workbook = Workbook(self.got_filename)

        worksheet1 = workbook.add_worksheet()
        workbook.add_worksheet("Data Sheet")
        worksheet3 = workbook.add_worksheet()

        bold = workbook.add_format({"bold": 1})

        worksheet1.write_string(0, 0, "Foo")
        worksheet1.write_number(1, 0, 123)

        worksheet3.write_string(1, 1, "Foo")
        worksheet3.write_string(2, 1, "Bar", bold)
        worksheet3.write_number(3, 2, 234)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_A1(self):
        """Test the creation of a simple workbook with A1 notation."""

        workbook = Workbook(self.got_filename)

        worksheet1 = workbook.add_worksheet()
        worksheet2 = workbook.add_worksheet("Data Sheet")
        worksheet3 = workbook.add_worksheet()

        bold = workbook.add_format({"bold": 1})

        worksheet1.write("A1", "Foo")
        worksheet1.write("A2", 123)

        worksheet3.write("B2", "Foo")
        worksheet3.write("B3", "Bar", bold)
        worksheet3.write("C4", 234)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_in_memory(self):
        """Test the creation of a simple workbook."""

        workbook = Workbook(self.got_filename, {"in_memory": True})

        worksheet1 = workbook.add_worksheet()
        workbook.add_worksheet("Data Sheet")
        worksheet3 = workbook.add_worksheet()

        bold = workbook.add_format({"bold": 1})

        worksheet1.write_string(0, 0, "Foo")
        worksheet1.write_number(1, 0, 123)

        worksheet3.write_string(1, 1, "Foo")
        worksheet3.write_string(2, 1, "Bar", bold)
        worksheet3.write_number(3, 2, 234)

        workbook.close()

        self.assertExcelEqual()



import sys
import os

extensions = ['sphinx.ext.intersphinx', 'sphinx.ext.extlinks']
source_suffix = '.rst'
master_doc = 'index'

project = u'XlsxWriter'
copyright = u'2013-2023, John McNamara'

version = '3.1.9'
release = version

exclude_patterns = []
pygments_style = 'sphinx'
intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}

sys.path.append(os.path.abspath('_themes'))
html_theme_path = ['_themes']
html_theme = 'bootstrap'
html_theme_options = {'nosidebar': True}
html_title = "XlsxWriter Documentation"
html_static_path = ['_static']
html_show_sphinx = True
html_show_copyright = True
htmlhelp_basename = 'XlsxWriterdoc'
html_add_permalinks = ""

latex_elements = {
    'pointsize': '11pt',
    'preamble': '',
}
latex_documents = [
    ('index', 'XlsxWriter.tex',
     'Creating Excel files with Python and XlsxWriter',
     'John McNamara', 'manual'),
]

latex_logo = '_images/logo.png'
man_pages = [
    ('index', 'xlsxwriter',
     'XlsxWriter Documentation',
     ['John McNamara'], 1)
]

texinfo_documents = [
    ('index',
     'XlsxWriter',
     'XlsxWriter Documentation',
     'John McNamara',
     'XlsxWriter',
     'Creating Excel files with Python and XlsxWriter',
     'Miscellaneous'),
]

epub_title = 'XlsxWriter'
epub_author = 'John McNamara'
epub_publisher = 'John McNamara'
epub_copyright = '2013-2023, John McNamara'

linkcheck_ignore = ["https://www.paypal.com"]

extlinks = {'issue': ('https://github.com/jmcnamara/XlsxWriter/issues/%s', 'Issue #'),
            'feature': ('https://github.com/jmcnamara/XlsxWriter/issues/%s', 'Feature Request #'),
            'pull': ('https://github.com/jmcnamara/XlsxWriter/pull/%s', 'Pull Request #')}



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...worksheet import Worksheet
from ...format import Format


class TestAssembleWorksheet(unittest.TestCase):
    """
    Test assembling a complete Worksheet file.

    """

    def test_assemble_xml_file(self):
        """Test writing a worksheet with row formatting set."""
        self.maxDiff = None

        fh = StringIO()
        worksheet = Worksheet()
        worksheet._set_filehandle(fh)
        cell_format = Format({"xf_index": 1})

        worksheet.set_row(1, 30)
        worksheet.set_row(3, None, None, {"hidden": 1})
        worksheet.set_row(6, None, cell_format)
        worksheet.set_row(9, 3)
        worksheet.set_row(12, 24, None, {"hidden": 1})
        worksheet.set_row(14, 0)

        worksheet.select()
        worksheet._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
                  <dimension ref="A2:A15"/>
                  <sheetViews>
                    <sheetView tabSelected="1" workbookViewId="0"/>
                  </sheetViews>
                  <sheetFormatPr defaultRowHeight="15"/>
                  <sheetData>
                    <row r="2" ht="30" customHeight="1"/>
                    <row r="4" hidden="1"/>
                    <row r="7" s="1" customFormat="1"/>
                    <row r="10" ht="3" customHeight="1"/>
                    <row r="13" ht="24" hidden="1" customHeight="1"/>
                    <row r="15" hidden="1"/>
                  </sheetData>
                  <pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>
                </worksheet>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("chart_size04.xlsx")

    def test_create_file(self):
        """Test XlsxWriter chartarea properties."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()
        chart = workbook.add_chart({"type": "column"})

        chart.axis_ids = [73773440, 73774976]

        data = [
            [1, 2, 3, 4, 5],
            [2, 4, 6, 8, 10],
            [3, 6, 9, 12, 15],
        ]

        worksheet.write_column("A1", data[0])
        worksheet.write_column("B1", data[1])
        worksheet.write_column("C1", data[2])

        chart.add_series({"values": "=Sheet1!$A$1:$A$5"})
        chart.add_series({"values": "=Sheet1!$B$1:$B$5"})
        chart.add_series({"values": "=Sheet1!$C$1:$C$5"})

        worksheet.insert_chart("E9", chart, {"x_offset": 8, "y_offset": 9})

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestCalculateSpans(unittest.TestCase):
    """
    Test the _prepare_formula Worksheet method for different formula types.

    """

    def test_prepare_formula(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

        self.worksheet.use_future_functions = True

        testcases = [
            ["=foo()", "foo()"],
            ["{foo()}", "foo()"],
            ["{=foo()}", "foo()"],
            # Dynamic functions.
            ["SEQUENCE(10)", "_xlfn.SEQUENCE(10)"],
            ["UNIQUES(A1:A10)", "UNIQUES(A1:A10)"],
            ["UUNIQUE(A1:A10)", "UUNIQUE(A1:A10)"],
            ["SINGLE(A1:A3)", "_xlfn.SINGLE(A1:A3)"],
            ["UNIQUE(A1:A10)", "_xlfn.UNIQUE(A1:A10)"],
            ["_xlfn.SEQUENCE(10)", "_xlfn.SEQUENCE(10)"],
            ["SORT(A1:A10)", "_xlfn._xlws.SORT(A1:A10)"],
            ["RANDARRAY(10,1)", "_xlfn.RANDARRAY(10,1)"],
            ["ANCHORARRAY(C1)", "_xlfn.ANCHORARRAY(C1)"],
            ["SORTBY(A1:A10,B1)", "_xlfn.SORTBY(A1:A10,B1)"],
            ["FILTER(A1:A10,1)", "_xlfn._xlws.FILTER(A1:A10,1)"],
            ["XMATCH(B1:B2,A1:A10)", "_xlfn.XMATCH(B1:B2,A1:A10)"],
            ["COUNTA(ANCHORARRAY(C1))", "COUNTA(_xlfn.ANCHORARRAY(C1))"],
            ["SEQUENCE(10)*SEQUENCE(10)", "_xlfn.SEQUENCE(10)*_xlfn.SEQUENCE(10)"],
            [
                'XLOOKUP("India",A22:A23,B22:B23)',
                '_xlfn.XLOOKUP("India",A22:A23,B22:B23)',
            ],
            [
                "XLOOKUP(B1,A1:A10,ANCHORARRAY(D1))",
                "_xlfn.XLOOKUP(B1,A1:A10,_xlfn.ANCHORARRAY(D1))",
            ],
            [
                "LAMBDA(_xlpm.number, _xlpm.number + 1)(1)",
                "_xlfn.LAMBDA(_xlpm.number, _xlpm.number + 1)(1)",
            ],
            # Newer dynamic functions (some duplicates with above).
            ["BYCOL(E1:G2)", "_xlfn.BYCOL(E1:G2)"],
            ["BYROW(E1:G2)", "_xlfn.BYROW(E1:G2)"],
            ["CHOOSECOLS(E1:G2,1)", "_xlfn.CHOOSECOLS(E1:G2,1)"],
            ["CHOOSEROWS(E1:G2,1)", "_xlfn.CHOOSEROWS(E1:G2,1)"],
            ["DROP(E1:G2,1)", "_xlfn.DROP(E1:G2,1)"],
            ["EXPAND(E1:G2,2)", "_xlfn.EXPAND(E1:G2,2)"],
            ["FILTER(E1:G2,H1:H2)", "_xlfn._xlws.FILTER(E1:G2,H1:H2)"],
            ["HSTACK(E1:G2)", "_xlfn.HSTACK(E1:G2)"],
            [
                "LAMBDA(_xlpm.number, _xlpm.number + 1)",
                "_xlfn.LAMBDA(_xlpm.number, _xlpm.number + 1)",
            ],
            [
                "MAKEARRAY(1,1,LAMBDA(_xlpm.row,_xlpm.col,TRUE)",
                "_xlfn.MAKEARRAY(1,1,_xlfn.LAMBDA(_xlpm.row,_xlpm.col,TRUE)",
            ],
            ["MAP(E1:G2,LAMBDA()", "_xlfn.MAP(E1:G2,_xlfn.LAMBDA()"],
            ["RANDARRAY(1)", "_xlfn.RANDARRAY(1)"],
            [
                'REDUCE("1,2,3",E1:G2,LAMBDA()',
                '_xlfn.REDUCE("1,2,3",E1:G2,_xlfn.LAMBDA()',
            ],
            ['SCAN("1,2,3",E1:G2,LAMBDA()', '_xlfn.SCAN("1,2,3",E1:G2,_xlfn.LAMBDA()'],
            ["SEQUENCE(E1:E2)", "_xlfn.SEQUENCE(E1:E2)"],
            ["SORT(F1)", "_xlfn._xlws.SORT(F1)"],
            ["SORTBY(E1:G1,E2:G2)", "_xlfn.SORTBY(E1:G1,E2:G2)"],
            ["SWITCH(WEEKDAY(E1)", "_xlfn.SWITCH(WEEKDAY(E1)"],
            ["TAKE(E1:G2,1)", "_xlfn.TAKE(E1:G2,1)"],
            ['TEXTSPLIT("foo bar", " ")', '_xlfn.TEXTSPLIT("foo bar", " ")'],
            ["TOCOL(E1:G1)", "_xlfn.TOCOL(E1:G1)"],
            ["TOROW(E1:E2)", "_xlfn.TOROW(E1:E2)"],
            ["UNIQUE(E1:G1)", "_xlfn.UNIQUE(E1:G1)"],
            ["VSTACK(E1:G2)", "_xlfn.VSTACK(E1:G2)"],
            ["WRAPCOLS(E1:F1,2)", "_xlfn.WRAPCOLS(E1:F1,2)"],
            ["WRAPROWS(E1:F1,2)", "_xlfn.WRAPROWS(E1:F1,2)"],
            ["XLOOKUP(M34,I35:I42,J35:K42)", "_xlfn.XLOOKUP(M34,I35:I42,J35:K42)"],
            # Future functions.
            ["COT()", "_xlfn.COT()"],
            ["CSC()", "_xlfn.CSC()"],
            ["IFS()", "_xlfn.IFS()"],
            ["LET()", "_xlfn.LET()"],
            ["PHI()", "_xlfn.PHI()"],
            ["RRI()", "_xlfn.RRI()"],
            ["SEC()", "_xlfn.SEC()"],
            ["XOR()", "_xlfn.XOR()"],
            ["ACOT()", "_xlfn.ACOT()"],
            ["BASE()", "_xlfn.BASE()"],
            ["COTH()", "_xlfn.COTH()"],
            ["CSCH()", "_xlfn.CSCH()"],
            ["DAYS()", "_xlfn.DAYS()"],
            ["IFNA()", "_xlfn.IFNA()"],
            ["SECH()", "_xlfn.SECH()"],
            ["ACOTH()", "_xlfn.ACOTH()"],
            ["BITOR()", "_xlfn.BITOR()"],
            ["F.INV()", "_xlfn.F.INV()"],
            ["GAMMA()", "_xlfn.GAMMA()"],
            ["GAUSS()", "_xlfn.GAUSS()"],
            ["IMCOT()", "_xlfn.IMCOT()"],
            ["IMCSC()", "_xlfn.IMCSC()"],
            ["IMSEC()", "_xlfn.IMSEC()"],
            ["IMTAN()", "_xlfn.IMTAN()"],
            ["MUNIT()", "_xlfn.MUNIT()"],
            ["SHEET()", "_xlfn.SHEET()"],
            ["T.INV()", "_xlfn.T.INV()"],
            ["VAR.P()", "_xlfn.VAR.P()"],
            ["VAR.S()", "_xlfn.VAR.S()"],
            ["ARABIC()", "_xlfn.ARABIC()"],
            ["BITAND()", "_xlfn.BITAND()"],
            ["BITXOR()", "_xlfn.BITXOR()"],
            ["CONCAT()", "_xlfn.CONCAT()"],
            ["F.DIST()", "_xlfn.F.DIST()"],
            ["F.TEST()", "_xlfn.F.TEST()"],
            ["IMAGE()", "_xlfn.IMAGE()"],
            ["IMCOSH()", "_xlfn.IMCOSH()"],
            ["IMCSCH()", "_xlfn.IMCSCH()"],
            ["IMSECH()", "_xlfn.IMSECH()"],
            ["IMSINH()", "_xlfn.IMSINH()"],
            ["MAXIFS()", "_xlfn.MAXIFS()"],
            ["MINIFS()", "_xlfn.MINIFS()"],
            ["SHEETS()", "_xlfn.SHEETS()"],
            ["SKEW.P()", "_xlfn.SKEW.P()"],
            ["SWITCH()", "_xlfn.SWITCH()"],
            ["T.DIST()", "_xlfn.T.DIST()"],
            ["T.TEST()", "_xlfn.T.TEST()"],
            ["Z.TEST()", "_xlfn.Z.TEST()"],
            ["XMATCH()", "_xlfn.XMATCH()"],
            ["COMBINA()", "_xlfn.COMBINA()"],
            ["DECIMAL()", "_xlfn.DECIMAL()"],
            ["RANK.EQ()", "_xlfn.RANK.EQ()"],
            ["STDEV.P()", "_xlfn.STDEV.P()"],
            ["STDEV.S()", "_xlfn.STDEV.S()"],
            ["UNICHAR()", "_xlfn.UNICHAR()"],
            ["UNICODE()", "_xlfn.UNICODE()"],
            ["BETA.INV()", "_xlfn.BETA.INV()"],
            ["F.INV.RT()", "_xlfn.F.INV.RT()"],
            ["ISO.CEILING()", "ISO.CEILING()"],
            ["NORM.INV()", "_xlfn.NORM.INV()"],
            ["RANK.AVG()", "_xlfn.RANK.AVG()"],
            ["T.INV.2T()", "_xlfn.T.INV.2T()"],
            ["TEXTJOIN()", "_xlfn.TEXTJOIN()"],
            ["TEXTJOIN()", "_xlfn.TEXTJOIN()"],
            ["AGGREGATE()", "_xlfn.AGGREGATE()"],
            ["BETA.DIST()", "_xlfn.BETA.DIST()"],
            ["BINOM.INV()", "_xlfn.BINOM.INV()"],
            ["BITLSHIFT()", "_xlfn.BITLSHIFT()"],
            ["BITRSHIFT()", "_xlfn.BITRSHIFT()"],
            ["CHISQ.INV()", "_xlfn.CHISQ.INV()"],
            ["ECMA.CEILING()", "ECMA.CEILING()"],
            ["F.DIST.RT()", "_xlfn.F.DIST.RT()"],
            ["FILTERXML()", "_xlfn.FILTERXML()"],
            ["GAMMA.INV()", "_xlfn.GAMMA.INV()"],
            ["ISFORMULA()", "_xlfn.ISFORMULA()"],
            ["MODE.MULT()", "_xlfn.MODE.MULT()"],
            ["MODE.SNGL()", "_xlfn.MODE.SNGL()"],
            ["NORM.DIST()", "_xlfn.NORM.DIST()"],
            ["PDURATION()", "_xlfn.PDURATION()"],
            ["T.DIST.2T()", "_xlfn.T.DIST.2T()"],
            ["T.DIST.RT()", "_xlfn.T.DIST.RT()"],
            ["WORKDAY.INTL()", "WORKDAY.INTL()"],
            ["ISOMITTED()", "_xlfn.ISOMITTED()"],
            ["TEXTAFTER()", "_xlfn.TEXTAFTER()"],
            ["BINOM.DIST()", "_xlfn.BINOM.DIST()"],
            ["CHISQ.DIST()", "_xlfn.CHISQ.DIST()"],
            ["CHISQ.TEST()", "_xlfn.CHISQ.TEST()"],
            ["EXPON.DIST()", "_xlfn.EXPON.DIST()"],
            ["FLOOR.MATH()", "_xlfn.FLOOR.MATH()"],
            ["GAMMA.DIST()", "_xlfn.GAMMA.DIST()"],
            ["ISOWEEKNUM()", "_xlfn.ISOWEEKNUM()"],
            ["NORM.S.INV()", "_xlfn.NORM.S.INV()"],
            ["WEBSERVICE()", "_xlfn.WEBSERVICE()"],
            ["TEXTBEFORE()", "_xlfn.TEXTBEFORE()"],
            ["ERF.PRECISE()", "_xlfn.ERF.PRECISE()"],
            ["FORMULATEXT()", "_xlfn.FORMULATEXT()"],
            ["LOGNORM.INV()", "_xlfn.LOGNORM.INV()"],
            ["NORM.S.DIST()", "_xlfn.NORM.S.DIST()"],
            ["NUMBERVALUE()", "_xlfn.NUMBERVALUE()"],
            ["QUERYSTRING()", "_xlfn.QUERYSTRING()"],
            ["ARRAYTOTEXT()", "_xlfn.ARRAYTOTEXT()"],
            ["VALUETOTEXT()", "_xlfn.VALUETOTEXT()"],
            ["CEILING.MATH()", "_xlfn.CEILING.MATH()"],
            ["CHISQ.INV.RT()", "_xlfn.CHISQ.INV.RT()"],
            ["CONFIDENCE.T()", "_xlfn.CONFIDENCE.T()"],
            ["COVARIANCE.P()", "_xlfn.COVARIANCE.P()"],
            ["COVARIANCE.S()", "_xlfn.COVARIANCE.S()"],
            ["ERFC.PRECISE()", "_xlfn.ERFC.PRECISE()"],
            ["FORECAST.ETS()", "_xlfn.FORECAST.ETS()"],
            ["HYPGEOM.DIST()", "_xlfn.HYPGEOM.DIST()"],
            ["LOGNORM.DIST()", "_xlfn.LOGNORM.DIST()"],
            ["PERMUTATIONA()", "_xlfn.PERMUTATIONA()"],
            ["POISSON.DIST()", "_xlfn.POISSON.DIST()"],
            ["QUARTILE.EXC()", "_xlfn.QUARTILE.EXC()"],
            ["QUARTILE.INC()", "_xlfn.QUARTILE.INC()"],
            ["WEIBULL.DIST()", "_xlfn.WEIBULL.DIST()"],
            ["CHISQ.DIST.RT()", "_xlfn.CHISQ.DIST.RT()"],
            ["FLOOR.PRECISE()", "_xlfn.FLOOR.PRECISE()"],
            ["NEGBINOM.DIST()", "_xlfn.NEGBINOM.DIST()"],
            ["NETWORKDAYS.INTL()", "NETWORKDAYS.INTL()"],
            ["PERCENTILE.EXC()", "_xlfn.PERCENTILE.EXC()"],
            ["PERCENTILE.INC()", "_xlfn.PERCENTILE.INC()"],
            ["CEILING.PRECISE()", "_xlfn.CEILING.PRECISE()"],
            ["CONFIDENCE.NORM()", "_xlfn.CONFIDENCE.NORM()"],
            ["FORECAST.LINEAR()", "_xlfn.FORECAST.LINEAR()"],
            ["GAMMALN.PRECISE()", "_xlfn.GAMMALN.PRECISE()"],
            ["PERCENTRANK.EXC()", "_xlfn.PERCENTRANK.EXC()"],
            ["PERCENTRANK.INC()", "_xlfn.PERCENTRANK.INC()"],
            ["BINOM.DIST.RANGE()", "_xlfn.BINOM.DIST.RANGE()"],
            ["FORECAST.ETS.STAT()", "_xlfn.FORECAST.ETS.STAT()"],
            ["FORECAST.ETS.CONFINT()", "_xlfn.FORECAST.ETS.CONFINT()"],
            ["FORECAST.ETS.SEASONALITY()", "_xlfn.FORECAST.ETS.SEASONALITY()"],
            ["Z.TEST(Z.TEST(Z.TEST()))", "_xlfn.Z.TEST(_xlfn.Z.TEST(_xlfn.Z.TEST()))"],
        ]

        for testcase in testcases:
            formula = testcase[0]
            exp = testcase[1]
            got = self.worksheet._prepare_formula(formula)

            self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...drawing import Drawing


class TestAssembleDrawing(unittest.TestCase):
    """
    Test assembling a complete Drawing file.

    """

    def test_assemble_xml_file(self):
        """Test writing a drawing with no cell data."""
        self.maxDiff = None

        fh = StringIO()
        drawing = Drawing()
        drawing._set_filehandle(fh)

        dimensions = [2, 1, 0, 0, 3, 6, 533257, 190357, 1219200, 190500, 0, 0]
        drawing_object = drawing._add_drawing_object()
        drawing_object["type"] = 2
        drawing_object["dimensions"] = dimensions
        drawing_object["width"] = 1142857
        drawing_object["height"] = 1142857
        drawing_object["description"] = "republic.png"
        drawing_object["shape"] = None
        drawing_object["anchor"] = 2
        drawing_object["rel_index"] = 1
        drawing_object["url_rel_index"] = 0
        drawing_object["tip"] = None

        drawing.embedded = 1

        drawing._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <xdr:wsDr xmlns:xdr="http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing" xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
                  <xdr:twoCellAnchor editAs="oneCell">
                    <xdr:from>
                      <xdr:col>2</xdr:col>
                      <xdr:colOff>0</xdr:colOff>
                      <xdr:row>1</xdr:row>
                      <xdr:rowOff>0</xdr:rowOff>
                    </xdr:from>
                    <xdr:to>
                      <xdr:col>3</xdr:col>
                      <xdr:colOff>533257</xdr:colOff>
                      <xdr:row>6</xdr:row>
                      <xdr:rowOff>190357</xdr:rowOff>
                    </xdr:to>
                    <xdr:pic>
                      <xdr:nvPicPr>
                        <xdr:cNvPr id="2" name="Picture 1" descr="republic.png"/>
                        <xdr:cNvPicPr>
                          <a:picLocks noChangeAspect="1"/>
                        </xdr:cNvPicPr>
                      </xdr:nvPicPr>
                      <xdr:blipFill>
                        <a:blip xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" r:embed="rId1"/>
                        <a:stretch>
                          <a:fillRect/>
                        </a:stretch>
                      </xdr:blipFill>
                      <xdr:spPr>
                        <a:xfrm>
                          <a:off x="1219200" y="190500"/>
                          <a:ext cx="1142857" cy="1142857"/>
                        </a:xfrm>
                        <a:prstGeom prst="rect">
                          <a:avLst/>
                        </a:prstGeom>
                      </xdr:spPr>
                    </xdr:pic>
                    <xdr:clientData/>
                  </xdr:twoCellAnchor>
                </xdr:wsDr>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)

    def test_assemble_xml_file_with_url(self):
        """Test writing a drawing with no cell data."""
        self.maxDiff = None

        fh = StringIO()
        drawing = Drawing()
        drawing._set_filehandle(fh)

        drawing = Drawing()
        drawing._set_filehandle(fh)

        dimensions = [2, 1, 0, 0, 3, 6, 533257, 190357, 1219200, 190500, 0, 0]
        drawing_object = drawing._add_drawing_object()
        drawing_object["type"] = 2
        drawing_object["dimensions"] = dimensions
        drawing_object["width"] = 1142857
        drawing_object["height"] = 1142857
        drawing_object["description"] = "republic.png"
        drawing_object["shape"] = None
        drawing_object["anchor"] = 2
        drawing_object["rel_index"] = 2
        drawing_object["url_rel_index"] = 1
        drawing_object["tip"] = "this is a tooltip"

        drawing.embedded = 1

        drawing._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <xdr:wsDr xmlns:xdr="http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing" xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
                  <xdr:twoCellAnchor editAs="oneCell">
                    <xdr:from>
                      <xdr:col>2</xdr:col>
                      <xdr:colOff>0</xdr:colOff>
                      <xdr:row>1</xdr:row>
                      <xdr:rowOff>0</xdr:rowOff>
                    </xdr:from>
                    <xdr:to>
                      <xdr:col>3</xdr:col>
                      <xdr:colOff>533257</xdr:colOff>
                      <xdr:row>6</xdr:row>
                      <xdr:rowOff>190357</xdr:rowOff>
                    </xdr:to>
                    <xdr:pic>
                    <xdr:nvPicPr>
                        <xdr:cNvPr id="2" name="Picture 1" descr="republic.png">
                          <a:hlinkClick xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" r:id="rId1" tooltip="this is a tooltip"/>
                        </xdr:cNvPr>
                        <xdr:cNvPicPr>
                            <a:picLocks noChangeAspect="1"/>
                        </xdr:cNvPicPr>
                    </xdr:nvPicPr>
                    <xdr:blipFill>
                        <a:blip xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" r:embed="rId2"/>
                        <a:stretch>
                            <a:fillRect/>
                        </a:stretch>
                    </xdr:blipFill>
                      <xdr:spPr>
                        <a:xfrm>
                          <a:off x="1219200" y="190500"/>
                          <a:ext cx="1142857" cy="1142857"/>
                        </a:xfrm>
                        <a:prstGeom prst="rect">
                          <a:avLst/>
                        </a:prstGeom>
                      </xdr:spPr>
                    </xdr:pic>
                    <xdr:clientData/>
                  </xdr:twoCellAnchor>
                </xdr:wsDr>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestExtractFilterTokens(unittest.TestCase):
    """
    Test the Worksheet _extract_filter_tokens() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_extract_filter_tokens(self):
        """Test the _extract_filter_tokens() method"""

        testcases = [
            [
                None,
                [],
            ],
            [
                "",
                [],
            ],
            [
                "0 <  2001",
                ["0", "<", "2001"],
            ],
            [
                "x <  2000",
                ["x", "<", "2000"],
            ],
            [
                "x >  2000",
                ["x", ">", "2000"],
            ],
            [
                "x == 2000",
                ["x", "==", "2000"],
            ],
            [
                "x >  2000 and x <  5000",
                ["x", ">", "2000", "and", "x", "<", "5000"],
            ],
            [
                'x = "goo"',
                ["x", "=", "goo"],
            ],
            [
                "x = moo",
                ["x", "=", "moo"],
            ],
            [
                'x = "foo baz"',
                ["x", "=", "foo baz"],
            ],
            [
                'x = "moo "" bar"',
                ["x", "=", 'moo " bar'],
            ],
            [
                'x = "foo bar" or x = "bar foo"',
                ["x", "=", "foo bar", "or", "x", "=", "bar foo"],
            ],
            [
                'x = "foo "" bar" or x = "bar "" foo"',
                ["x", "=", 'foo " bar', "or", "x", "=", 'bar " foo'],
            ],
            [
                'x = """"""""',
                ["x", "=", '"""'],
            ],
            [
                "x = Blanks",
                ["x", "=", "Blanks"],
            ],
            [
                "x = NonBlanks",
                ["x", "=", "NonBlanks"],
            ],
            [
                "top 10 %",
                ["top", "10", "%"],
            ],
            [
                "top 10 items",
                ["top", "10", "items"],
            ],
        ]

        for testcase in testcases:
            expression = testcase[0]

            exp = testcase[1]
            got = self.worksheet._extract_filter_tokens(expression)

            self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("comment05.xlsx")

        # It takes about 50 times longer to run this test with this file
        # included. Only turn it on for pre-release testing.
        self.ignore_files = ["xl/drawings/vmlDrawing1.vml"]

    def test_create_file(self):
        """
        Test the creation of a simple XlsxWriter file with comments.
        Test the VML data and shape ids for blocks of comments > 1024.
        """

        workbook = Workbook(self.got_filename)

        worksheet1 = workbook.add_worksheet()
        worksheet2 = workbook.add_worksheet()
        worksheet3 = workbook.add_worksheet()

        for row in range(0, 127 + 1):
            for col in range(0, 15 + 1):
                worksheet1.write_comment(row, col, "Some text")

        worksheet3.write_comment("A1", "More text")

        worksheet1.set_comments_author("John")
        worksheet3.set_comments_author("John")

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...contenttypes import ContentTypes


class TestAssembleContentTypes(unittest.TestCase):
    """
    Test assembling a complete ContentTypes file.

    """

    def test_assemble_xml_file(self):
        """Test writing an ContentTypes file."""
        self.maxDiff = None

        fh = StringIO()
        content = ContentTypes()
        content._set_filehandle(fh)

        content._add_worksheet_name("sheet1")
        content._add_default(("jpeg", "image/jpeg"))
        content._add_shared_strings()
        content._add_calc_chain()

        content._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <Types xmlns="http://schemas.openxmlformats.org/package/2006/content-types">

                  <Default Extension="rels" ContentType="application/vnd.openxmlformats-package.relationships+xml"/>
                  <Default Extension="xml" ContentType="application/xml"/>
                  <Default Extension="jpeg" ContentType="image/jpeg"/>

                  <Override PartName="/docProps/app.xml" ContentType="application/vnd.openxmlformats-officedocument.extended-properties+xml"/>
                  <Override PartName="/docProps/core.xml" ContentType="application/vnd.openxmlformats-package.core-properties+xml"/>
                  <Override PartName="/xl/styles.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml"/>
                  <Override PartName="/xl/theme/theme1.xml" ContentType="application/vnd.openxmlformats-officedocument.theme+xml"/>
                  <Override PartName="/xl/workbook.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml"/>
                  <Override PartName="/xl/worksheets/sheet1.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml"/>
                  <Override PartName="/xl/sharedStrings.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml"/>
                  <Override PartName="/xl/calcChain.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.calcChain+xml"/>
                </Types>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...table import Table


class TestWriteAutoFilter(unittest.TestCase):
    """
    Test the Table _write_auto_filter() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.table = Table()
        self.table._set_filehandle(self.fh)

    def test_write_auto_filter(self):
        """Test the _write_auto_filter() method"""

        self.table.properties["autofilter"] = "C3:F13"

        self.table._write_auto_filter()

        exp = """<autoFilter ref="C3:F13"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
import warnings
from ...utility import xl_col_to_name


class TestUtility(unittest.TestCase):
    """
    Test xl_col_to_name() utility function.

    """

    def test_xl_col_to_name(self):
        """Test xl_col_to_name()"""

        tests = [
            # col,  col string
            (0, "A"),
            (1, "B"),
            (2, "C"),
            (9, "J"),
            (24, "Y"),
            (25, "Z"),
            (26, "AA"),
            (254, "IU"),
            (255, "IV"),
            (256, "IW"),
            (16383, "XFD"),
            (16384, "XFE"),
            (-1, None),
        ]

        for col, string in tests:
            exp = string
            got = xl_col_to_name(col)

            # Ignore the warnings for negative values.
            warnings.filterwarnings("ignore")

            self.assertEqual(got, exp)

    def test_xl_col_to_name_abs(self):
        """Test xl_col_to_name() with absolute references"""

        tests = [
            # col, col_abs, col string
            (0, True, "$A"),
            (-1, True, None),
        ]

        for col, col_abs, string in tests:
            exp = string
            got = xl_col_to_name(col, col_abs)

            # Ignore the warnings for negative values.
            warnings.filterwarnings("ignore")

            self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("protect08.xlsx")

    def test_create_file(self):
        """Test the a simple XlsxWriter file with worksheet protection."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        options = {
            "objects": True,
            "scenarios": True,
            "format_cells": True,
            "format_columns": True,
            "format_rows": True,
            "insert_columns": True,
            "insert_rows": True,
            "insert_hyperlinks": True,
            "delete_columns": True,
            "delete_rows": True,
            "select_locked_cells": False,
            "sort": True,
            "autofilter": True,
            "pivot_tables": True,
            "select_unlocked_cells": False,
        }

        worksheet.protect("", options)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
import os
from .helperfunctions import _compare_xlsx_files


class ExcelComparisonTest(unittest.TestCase):
    """
    Test class for comparing a file created by XlsxWriter against a file
    created by Excel.

    """

    def set_filename(self, filename):
        # Set the filename and paths for the test xlsx files.
        self.maxDiff = None
        self.got_filename = ""
        self.exp_filename = ""
        self.ignore_files = []
        self.ignore_elements = {}
        self.test_dir = "xlsxwriter/test/comparison/"
        self.vba_dir = self.test_dir + "xlsx_files/"
        self.image_dir = self.test_dir + "images/"

        # The reference Excel generated file.
        self.exp_filename = self.test_dir + "xlsx_files/" + filename

        # The generated XlsxWriter file.
        self.got_filename = self.test_dir + "_test_" + filename

    def set_text_file(self, filename):
        # Set the filename and path for text files used in tests.
        self.txt_filename = self.test_dir + "xlsx_files/" + filename

    def assertExcelEqual(self):
        # Compare the generate file and the reference Excel file.
        got, exp = _compare_xlsx_files(
            self.got_filename,
            self.exp_filename,
            self.ignore_files,
            self.ignore_elements,
        )

        self.assertEqual(exp, got)

    def tearDown(self):
        # Cleanup by removing the temp excel file created for testing.
        if os.path.exists(self.got_filename):
            os.remove(self.got_filename)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from ...utility import xl_cell_to_rowcol


class TestUtility(unittest.TestCase):
    """
    Test xl_cell_to_rowcol() utility function.

    """

    def test_xl_cell_to_rowcol(self):
        """Test xl_cell_to_rowcol()"""

        tests = [
            # row, col, A1 string
            (0, 0, "A1"),
            (0, 1, "B1"),
            (0, 2, "C1"),
            (0, 9, "J1"),
            (1, 0, "A2"),
            (2, 0, "A3"),
            (9, 0, "A10"),
            (1, 24, "Y2"),
            (7, 25, "Z8"),
            (9, 26, "AA10"),
            (1, 254, "IU2"),
            (1, 255, "IV2"),
            (1, 256, "IW2"),
            (0, 16383, "XFD1"),
            (1048576, 16384, "XFE1048577"),
        ]

        for row, col, string in tests:
            exp = (row, col)
            got = xl_cell_to_rowcol(string)
            self.assertEqual(got, exp)

    def test_xl_cell_to_rowcol_abs(self):
        """Test xl_cell_to_rowcol() with absolute references"""

        tests = [
            # row, col, row_abs, col_abs, A1 string
            (0, 0, 0, 0, "A1"),
            (0, 0, 1, 0, "A$1"),
            (0, 0, 0, 1, "$A1"),
            (0, 0, 1, 1, "$A$1"),
        ]

        for row, col, row_abs, col_abs, string in tests:
            exp = (row, col)
            got = xl_cell_to_rowcol(string)
            self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestPixelsToRowCol(unittest.TestCase):
    """
    Test the Worksheet _pixels_to_xxx() methods.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    # Function for testing.
    def width_to_pixels(self, width):
        max_digit_width = 7
        padding = 5

        if width < 1:
            pixels = int(width * (max_digit_width + padding) + 0.5)
        else:
            pixels = int(width * max_digit_width + 0.5) + padding

        return pixels

    # Function for testing.
    def height_to_pixels(self, height):
        return int(4.0 / 3.0 * height)

    def test_pixels_to_width(self):
        """Test the _pixels_to_width() function"""

        for pixels in range(1791):
            exp = pixels
            got = self.width_to_pixels(self.worksheet._pixels_to_width(pixels))

            self.assertEqual(got, exp)

    def test_pixels_to_height(self):
        """Test the _pixels_to_height() function"""

        for pixels in range(546):
            exp = pixels
            got = self.height_to_pixels(self.worksheet._pixels_to_height(pixels))

            self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("set_column01.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        worksheet.set_column_pixels("A:A", 1)
        worksheet.set_column_pixels("B:B", 2)
        worksheet.set_column_pixels("C:C", 3)
        worksheet.set_column_pixels("D:D", 4)
        worksheet.set_column_pixels("E:E", 5)
        worksheet.set_column_pixels("F:F", 6)
        worksheet.set_column_pixels("G:G", 7)
        worksheet.set_column_pixels("H:H", 8)
        worksheet.set_column_pixels("I:I", 9)
        worksheet.set_column_pixels("J:J", 10)
        worksheet.set_column_pixels("K:K", 11)
        worksheet.set_column_pixels("L:L", 12)
        worksheet.set_column_pixels("M:M", 13)
        worksheet.set_column_pixels("N:N", 14)
        worksheet.set_column_pixels("O:O", 15)
        worksheet.set_column_pixels("P:P", 16)
        worksheet.set_column_pixels("Q:Q", 17)
        worksheet.set_column_pixels("R:R", 18)
        worksheet.set_column_pixels("S:S", 19)
        worksheet.set_column_pixels("T:T", 20)
        worksheet.set_column_pixels("U:U", 21)
        worksheet.set_column_pixels("V:V", 22)
        worksheet.set_column_pixels("W:W", 23)
        worksheet.set_column_pixels("X:X", 24)
        worksheet.set_column_pixels("Y:Y", 25)
        worksheet.set_column_pixels("Z:Z", 26)
        worksheet.set_column_pixels("AB:AB", 65)
        worksheet.set_column_pixels("AC:AC", 66)
        worksheet.set_column_pixels("AD:AD", 67)
        worksheet.set_column_pixels("AE:AE", 68)
        worksheet.set_column_pixels("AF:AF", 69)
        worksheet.set_column_pixels("AG:AG", 70)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...sharedstrings import SharedStringTable
from ...sharedstrings import SharedStrings


class TestAssembleSharedStrings(unittest.TestCase):
    """
    Test assembling a complete SharedStrings file.

    """

    def test_assemble_xml_file(self):
        """Test the _assemble_xml_file() method"""

        string_table = SharedStringTable()

        # Add some strings with leading/trailing whitespace.
        index = string_table._get_shared_string_index("abcdefg")
        self.assertEqual(index, 0)

        index = string_table._get_shared_string_index("   abcdefg")
        self.assertEqual(index, 1)

        index = string_table._get_shared_string_index("abcdefg   ")
        self.assertEqual(index, 2)

        string_table._sort_string_data()

        fh = StringIO()
        sharedstrings = SharedStrings()
        sharedstrings._set_filehandle(fh)
        sharedstrings.string_table = string_table

        sharedstrings._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <sst xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" count="3" uniqueCount="3">
                  <si>
                    <t>abcdefg</t>
                  </si>
                  <si>
                    <t xml:space="preserve">   abcdefg</t>
                  </si>
                  <si>
                    <t xml:space="preserve">abcdefg   </t>
                  </si>
                </sst>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)
